__version__ = '0.0.4'
__title__ = 'zwdb'
__description__ = 'Personal Database Wrapper.'
__url__ = 'https://github.com/njzhaowei'
__author__ = 'Zhao Wei'
__author_email__ = 'yewberry@163.com'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2020 Zhao Wei'